#ifndef __CZ8RL1_H__
#define __CZ8RL1_H__

/* X1->CZ8RL1 command & CZ8RL1->X1 status */
#define CZ8RL1_EJECT  0x00
#define CZ8RL1_STOP   0x01
#define CZ8RL1_PLAY   0x02
#define CZ8RL1_FF     0x03
#define CZ8RL1_REW    0x04
#define CZ8RL1_AFF    0x05
#define CZ8RL1_AREW   0x06
#define CZ8RL1_REC    0x0A

/* X1->CZ8RL1 command (bit1..6 = Don't cate) */
#define CZ8RL1_STATUS 0x80
#define CZ8RL1_SENSOR 0x81

/* CZ8RL1->X1 status */
#define CZ8RL1_BREAK  0xff
/* 0x80`0x87 sensor (answer of CZ8RL1_SENSOR) */
/*             bit0   :tape end   (0=ON)       */
/*             bit1   :tape set   (1=ON)       */
/*             bit2   :writeprtect(0=ON)       */

/* function return code */
#define CZ8RL1_STS_OK           0x100 /* G[                             */
#define CZ8RL1_STS_AUTOSTOP     0x101 /* ~                               */
#define CZ8RL1_STS_BUFFULL      0x102 /* obt@t                           */
#define CZ8RL1_STS_BREAK        0x103 /* JZbg~iatrxo)            */
#define CZ8RL1_STS_DOAPSS       0x104 /* `orrs                         */
/* function error code */
#define CZ8RL1_ERR_NOTFOUND  -1 /* CZ8RL1(BUSY=H , STATUS=H)          */
#define CZ8RL1_ERR_BSYACK    -2 /* BUSYK莞ԓHIGHɉȂ */
#define CZ8RL1_ERR_DOAPSS    -3 /* `orrs                         */
#define CZ8RL1_ERR_NOHEADER  -4 /* STATUS̃wb_[M               */
#define CZ8RL1_ERR_BADHEAD   -5 /* STATUS̃wb_[ɒKO         */
#define CZ8RL1_ERR_BADBIT    -6 /* f[^rbg̒KO             */

#define CZ8RL1_ERR_SPEED1   -10/* botxǂȂ               */
#define CZ8RL1_ERR_SPEED2   -11/* botxǂȂ               */

/* command output to CZ8RL1            */
/* in     : command code               */
/* return : status code or error value */
int cz8rl1_write(BYTE value);
#define cz8rl1_get_status() cz8rl1_write(0x80)
#define cz8rl1_get_sensor() cz8rl1_write(0x81)

/* sampling CZ8RL1 read data */
int CZ8RL1_read_data(
	BYTE *buf,			/* buffer pointer                  */
	DWORD *pbufsize,	/* buffer size / sampling size     */
	DWORD *numeddge,	/*             / num of eddges     */
	int busybreak,		/* break when busy high            */
	int rate,			/* samplingrate                    */
	double *jitterl,	/* jitter fall  % / jitter avg.    */
	double *jitterh,	/* jitter raise % / jitter avg.    */
	int idlebreak);		/* auto ditect stop time           */

int cz8rl1_write_data(BYTE *buf,DWORD *pbufsize,DWORD rate,int busybreak);

/* initialize CZ8RL1 I/O port */
int CZ8RL1_init(
	WORD wport_addr,	/* ݃|[g̃AhX							*/
	BYTE wdata_base,		/* |[g̏li܂ޕɐM̐ݒj		*/
	BYTE mask_write_data,	/* WRITE_DATAo͐Mrbg̃}XNl			*/
	BYTE mask_command,		/* COMMANDo͐Mrbg̃}XNl				*/
	BYTE mask_strobe,		/* STROBEo͐Mrbg̃}XNl				*/
	WORD rport_addr,	/* ǂݍ݃|[g̃AhX							*/
	BYTE rdata_base,		/* |[g̕ɐݒ}XN						*/
	BYTE mask_busy,			/* BUSY͐Mrbg̃}XNl					*/
	BYTE mask_status,		/* STATUS͐Mrbg̃}XNl				*/
	BYTE mask_read_data		/* READ_DATA͐Mrbg̃}XNl			*/
);
#endif
